from ._functions import (correct_magnitudes,
                         create_colors,
                         create_combinations,
                         assemble_work_df,
                         calculate_abs_mag)